package com.terraocean.plugin.bridge

import com.terraocean.plugin.Settings
import com.terraocean.plugin.Strings
import com.terraocean.plugin.instance
import hazae41.minecraft.kutils.bukkit.info
import hazae41.minecraft.kutils.bukkit.warning
import okhttp3.*
import org.bukkit.Bukkit
import java.util.concurrent.TimeUnit

val client = OkHttpClient.Builder()
    .readTimeout(0, TimeUnit.MILLISECONDS)
    .build()

lateinit var socket: WebSocket

fun establishConnection() {
    instance.info("establishing connection to ${Settings.socketURL}...")
    val request = Request.Builder()
        .url(Settings.socketURL)
        .build()

    socket = client.newWebSocket(request, WSListener())
}

fun processMessage(msg: List<String>) {
    when (msg[0]) {
        "leave" -> {
            Bukkit.getPlayer(msg[1])?.kickPlayer(Strings.kickVoiceChannel)
        }
        "vote" -> {
            Bukkit.getOnlinePlayers().forEach {
                it.sendMessage(Strings.voteNew)
            }
        }
        "votewarn" -> {
            Bukkit.getPlayer(msg[1])?.sendMessage(Strings.voteWarn)
        }
        "votetimeout" -> {
            Bukkit.getPlayer(msg[1])?.kickPlayer(Strings.voteTimeout)
        }
        "kick" -> {
            Bukkit.getPlayer(msg[1])?.kickPlayer(Strings.voteKick)
        }
        "whitelistadd" -> {
           instance.server.run {
                getOfflinePlayer(msg[1]).isWhitelisted = true
            }
        }
        "whitelistremove" -> {
            instance.server.run {
                getPlayer(msg[1])?.kickPlayer(Strings.kickRemovedFromWhitelist)
                getOfflinePlayer(msg[1]).isWhitelisted = false
            }
        }
        else -> instance.warning(
            "Potentially out of date plugin. Unknown message: " + msg.joinToString(" ")
        )
    }
}

class WSListener: WebSocketListener() {
    override fun onMessage(webSocket: WebSocket, text: String) {
        processMessage(text.split(' '))
    }

    override fun onOpen(webSocket: WebSocket, response: Response) {
        instance.info("Socket connection established to $response.")
    }

    override fun onFailure(webSocket: WebSocket, t: Throwable, response: Response?) {
        TODO("Automatic reconnection")
    }
}